@@ -33,6 +33,9 @@
 #include <stdarg.h>
 #include <errno.h>
 #include <byteswap.h>
+#ifdef HAVE_SYSCALL_H
+#include <sys/syscall.h>
+#endif
 #ifdef HAVE_LINUX_SECUREBITS_H
 #include <linux/securebits.h>
 #endif
@@ -102,7 +105,6 @@ extern int capget(cap_user_header_t header, const cap_user_data_t data);
 #define SECURE_NO_SETUID_FIXUP_LOCKED   3  /* make bit-2 immutable */
 #endif
 
-
 // States: new, allocated, initted, updated, applied
 typedef enum { CAPNG_NEW, CAPNG_ERROR, CAPNG_ALLOCATED, CAPNG_INIT,
 	CAPNG_UPDATED, CAPNG_APPLIED } capng_states_t;
@@ -149,7 +151,11 @@ static void init(void)
 	}
 
 	memset(&m.data, 0, sizeof(cap_data_t));
-	m.hdr.pid = getpid();
+#ifdef HAVE_SYSCALL_H
+	m.hdr.pid = (unsigned)syscall(__NR_gettid);
+#else
+	m.hdr.pid = (unsigned)getpid();
+#endif
 	m.state = CAPNG_ALLOCATED;
 }
 
@@ -217,7 +223,11 @@ static int get_bounding_set(void)
 	FILE *f;
 
 	snprintf(buf, sizeof(buf), "/proc/%u/status", m.hdr.pid ? m.hdr.pid :
-		getpid());
+#ifdef HAVE_SYSCALL_H
+		(unsigned)syscall(__NR_gettid));
+#else
+		(unsigned)getpid();
+#endif
 	f = fopen(buf, "r");
 	if (f == NULL)
 		return -1;
@@ -287,8 +297,10 @@ static int load_data(const struct vfs_cap_data *filedata, int size)
 	m.data.v3[0].inheritable = FIXUP(filedata->data[0].inheritable);
 	m.data.v3[1].inheritable = FIXUP(filedata->data[1].inheritable);
 	if (magic & VFS_CAP_FLAGS_EFFECTIVE) {
-		m.data.v3[0].effective = 0xFFFFFFFFU;
-		m.data.v3[1].effective = 0xFFFFFFFFU;
+		m.data.v3[0].effective =
+			m.data.v3[0].permitted | m.data.v3[0].inheritable;
+		m.data.v3[1].effective =
+			m.data.v3[1].permitted | m.data.v3[1].inheritable;
 	} else {
 		m.data.v3[0].effective = 0;
 		m.data.v3[1].effective = 0;
