@@ -1,6 +1,6 @@
 /*
  * netcap.c - A program that lists network apps with capabilities
- * Copyright (c) 2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright (c) 2009-10 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This software may be freely redistributed and/or modified under the
@@ -207,14 +207,14 @@ static int collect_process_info(void)
 	return 0;
 }
 
-static void report_finding(int port, const char *type)
+static void report_finding(int port, const char *type, const char *ifc)
 {
 	struct passwd *p;
 	lnode *n = list_get_cur(&l);
 		
 	// And print out anything with capabilities
 	if (header == 0) {
-		printf("%-5s %-5s %-10s %-16s %-4s%5s  %s\n",
+		printf("%-5s %-5s %-10s %-16s %-4s %-6s %s\n",
 			"ppid", "pid", "acct", "command", "type", "port", 
 			"capabilities");
 			header = 1;
@@ -235,8 +235,12 @@ static void report_finding(int port, const char *type)
 		printf("%-5d %-5d %-10s", n->ppid, n->pid, tacct);
 	} else
 		printf("%-5d %-5d %-10d", n->ppid, n->pid, last_uid);
-	printf(" %-16s %-4s %-5d %s %s\n", n->cmd, type, port, n->capabilities,
-		n->bounds);
+	printf(" %-16s %-4s", n->cmd, type);
+	if (ifc)
+		printf(" %-6s", ifc);
+	else
+		printf(" %-6d", port);
+	printf(" %s %s\n", n->capabilities, n->bounds);
 }
 
 static void read_tcp(const char *proc, const char *type)
@@ -268,7 +272,7 @@ static void read_tcp(const char *proc, const char *type)
 			&state, &txq, &rxq, &timer_run, &time_len, &retr,
 			&uid, &timeout, &inode, more);
 		if (list_find_inode(&l, inode))
-			report_finding(local_port, type);
+			report_finding(local_port, type, NULL);
 	}
 	fclose(f);
 }
@@ -302,7 +306,7 @@ static void read_udp(const char *proc, const char *type)
 			&state, &txq, &rxq, &timer_run, &time_len, &retr,
 			&uid, &timeout, &inode, more);
 		if (list_find_inode(&l, inode))
-			report_finding(local_port, type);
+			report_finding(local_port, type, NULL);
 	}
 	fclose(f);
 }
@@ -336,20 +340,82 @@ static void read_raw(const char *proc, const char *type)
 			&state, &txq, &rxq, &timer_run, &time_len, &retr,
 			&uid, &timeout, &inode, more);
 		if (list_find_inode(&l, inode))
-			report_finding(0, type);
+			report_finding(0, type, NULL);
+	}
+	fclose(f);
+}
+
+// Caller must have buffer > 16 bytes
+static void get_interface(unsigned int iface, char *ifc)
+{
+	unsigned int line = 0;
+	FILE *f;
+	char buf[256], more[256];
+
+	// Terminate the interface in case of error
+	*ifc = 0;
+
+	// Increment the interface number since header is 2 lines long
+	iface++;
+
+	f = fopen("/proc/net/dev", "rt");
+	if (f == NULL) {
+		if (errno != ENOENT)
+			fprintf(stderr, "Can't open /proc/net/dev: %s\n",
+				strerror(errno));
+		return;
+	}
+	__fsetlocking(f, FSETLOCKING_BYCALLER);
+	while (fgets(buf, sizeof(buf), f)) {
+		if (line == iface) {
+			char *c;
+			sscanf(buf, "%16s: %256s\n", ifc, more);
+			c = strchr(ifc, ':');
+			if (c)
+				*c = 0;
+			fclose(f);
+			return;
+		}
+		line++;
+	}
+	fclose(f);
+}
+
+static void read_packet(void)
+{
+	int line = 0;
+	FILE *f;
+	char buf[256];
+	unsigned long sk, inode;
+	unsigned int ref_cnt, type, proto, iface, r, rmem, uid;
+	char more[256], ifc[32];
+
+	f = fopen("/proc/net/packet", "rt");
+	if (f == NULL) {
+		if (errno != ENOENT)
+			fprintf(stderr, "Can't open /proc/net/packet: %s\n",
+				strerror(errno));
+		return;
+	}
+	__fsetlocking(f, FSETLOCKING_BYCALLER);
+	while (fgets(buf, sizeof(buf), f)) {
+		if (line == 0) {
+			line++;
+			continue;
+		}
+		more[0] = 0;
+		sscanf(buf, "%lX %u %u %X %u %u %u %u %lu %256s\n",
+			&sk, &ref_cnt, &type, &proto, &iface,
+			&r, &rmem, &uid, &inode, more);
+		get_interface(iface, ifc);
+		if (list_find_inode(&l, inode))
+			report_finding(0, "pkt", ifc);
 	}
 	fclose(f);
 }
 
 int main(int argc, char __attribute__((unused)) *argv[])
 {
-	int line = 0;
-	FILE *f;
-	char buf[256];
-	unsigned long inode;
-	int d, uid;
-	char rem_addr[128], more[512];
-
 	if (argc > 1) {
 		fputs("Too many arguments\n", stderr);
 		usage();
@@ -370,29 +436,9 @@ int main(int argc, char __attribute__((unused)) *argv[])
 	read_raw("/proc/net/raw", "raw");
 	read_raw("/proc/net/raw6", "raw6");
 
-	// And last get packet capture sockets
-	line = 0;
-	f = fopen("/proc/net/packet", "rt");
-	if (f == NULL) {
-		if (errno != ENOENT)
-			fprintf(stderr, "Can't open /proc/net/packet: %s\n", 
-				strerror(errno));
-		goto end;
-	}
-	__fsetlocking(f, FSETLOCKING_BYCALLER);
-	while (fgets(buf, sizeof(buf), f)) {
-		if (line == 0) {
-			line++;
-			continue;
-		}
-		more[0] = 0;
-		sscanf(buf, "%32[0-9A-Fa-f] %d %d %X %d %d %d %d %lu %512s\n",
-			rem_addr, &d, &d, &d, &d, &d, &d, &uid, &inode, more);
-		if (list_find_inode(&l, inode))
-			report_finding(0, "pkt");
-	}
-	fclose(f);
-end:
+	// And last, read packet sockets
+	read_packet();
+
 	list_clear(&l);
 	return 0;
 }
